@@ -37,16 +37,12 @@ esac
 cd $NEUTRON_VPNAAS_DIR
 sudo chown -R $owner:stack $NEUTRON_VPNAAS_DIR
 
-# Run tests
 echo "Running neutron $VENV test suite"
 set +e
 sudo -H -u $owner $sudo_env tox -e $VENV
 testr_exit_code=$?
 set -e
 
-echo "Dumping log from tox_install.sh"
-cat /tmp/tox_install.txt
-
 # Collect and parse results
 generate_testr_results
 exit $testr_exit_code
